window.Gleam.initSearch([{"doc":"cel","title":"cel","content":"# Common Expression Language (Gleam)\n\n[![Package Version](https://img.shields.io/hexpm/v/cel)](https://hex.pm/packages/cel)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/cel/)\n\nThe [Common Expression Language (CEL)](https://github.com/google/cel-spec) is a non-Turing complete\nlanguage designed for simplicity, speed, safety, and portability.\nCEL&#39;s C-like syntax looks nearly identical to equivalent expressions in C++, Go, Java, and TypeScript.\nCEL is ideal for lightweight expression evaluation when a fully sandboxed scripting language is too resource intensive.\n\n```java\n// Check whether a resource name starts with a group name.\nresource.name.startsWith(&quot;/groups/&quot; + auth.claims.group)\n```\n\n```go\n// Determine whether the request is in the permitted time window.\nrequest.time - resource.age &lt; duration(&quot;24h&quot;)\n```\n\n```typescript\n// Check whether all resource names in a list match a given filter.\nauth.claims.email_verified &amp;&amp; resources.all(r, r.startsWith(auth.claims.email))\n```\n\n## Usage\n\n```sh\ngleam add cel@0.3\n```\n\n```gleam\nimport gleam/io\n\nimport cel/interpreter\nimport cel/interpreter/context\nimport cel/interpreter/value\n\npub fn main() {\n  let source =\n    &quot;[a, b, c].map(x, x + 2).filter(x, x &gt; 4)[0] == 5 ? &#39;wibble&#39; : &#39;wobble&#39;&quot;\n\n  let ctx =\n    interpreter.default_context()\n    |&gt; context.insert_variable(&quot;a&quot;, value.Int(1))\n    |&gt; context.insert_variable(&quot;b&quot;, value.Int(3))\n    |&gt; context.insert_variable(&quot;c&quot;, value.Int(5))\n\n  let assert Ok(program) = interpreter.new(source)\n  let assert Ok(value.String(answer)) = interpreter.execute(program, ctx)\n\n  io.println(&quot;execution result: &quot; &lt;&gt; answer)\n  // execution result: wibble\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/cel&gt;.\n\n## Development\n\n```sh\ngleam test  # Run the tests\n```\n\n## State\n\nThis library is still very early in its development. The current state and planned future work:\n- [x] Lexing\n- [x] Parsing\n- [x] Expression evaluation\n- [x] Variable context resolution\n- [x] Functions/macros\n- [x] Context provision through Dynamic\n- [ ] Value → JSON\n- [ ] Type checking\n- [ ] Field inits (`ident{&quot;a&quot;: 5}`)\n- [ ] Duration + Timestamp\n- [ ] (Maybe) Serializable AST\n","url":"index.html"},{"doc":"cel/parser/lexer","title":"Lexer","content":"pub opaque type Lexer\n\n\nSynonyms:\ncel/parser/lexer.Lexer\ncel/parser/lexer Lexer","url":"cel/parser/lexer.html#Lexer"},{"doc":"cel/parser/lexer","title":"Position","content":"pub type Position {\n  Position(byte_offset: Int, byte_size: Int)\n}\n\nPosition(byte_offset: Int, byte_size: Int)\n\n\nSynonyms:\ncel/parser/lexer.Position\ncel/parser/lexer Position","url":"cel/parser/lexer.html#Position"},{"doc":"cel/parser/lexer","title":"Token","content":"pub type Token {\n  Ident(String)\n  Int(String)\n  UInt(String)\n  Float(String)\n  String(String)\n  Bytes(BitArray)\n  Bool(Bool)\n  Null\n  Plus\n  Minus\n  Star\n  Slash\n  Percent\n  LessThan\n  LessThanEq\n  GreaterThan\n  GreaterThanEq\n  Equals\n  NotEquals\n  In\n  Or\n  And\n  LeftParen\n  RightParen\n  LeftCurly\n  RightCurly\n  LeftSquare\n  RightSquare\n  At\n  Dot\n  Colon\n  SemiColon\n  Comma\n  ExclamationMark\n  Hash\n  QuestionMark\n  Comment(String)\n  Whitespace(String)\n  EndOfFile\n  Reserved(String)\n  UnterminatedString(String)\n  UnterminatedBytes(BitArray)\n  UnexpectedGrapheme(String)\n  InvalidByteLiteral(String)\n}\n\nIdent(String)\n\n\nInt(String)\n\n\nUInt(String)\n\n\nFloat(String)\n\n\nString(String)\n\n\nBytes(BitArray)\n\n\nBool(Bool)\n\n\nNull\n\n\nPlus\n\n\nMinus\n\n\nStar\n\n\nSlash\n\n\nPercent\n\n\nLessThan\n\n\nLessThanEq\n\n\nGreaterThan\n\n\nGreaterThanEq\n\n\nEquals\n\n\nNotEquals\n\n\nIn\n\n\nOr\n\n\nAnd\n\n\nLeftParen\n\n\nRightParen\n\n\nLeftCurly\n\n\nRightCurly\n\n\nLeftSquare\n\n\nRightSquare\n\n\nAt\n\n\nDot\n\n\nColon\n\n\nSemiColon\n\n\nComma\n\n\nExclamationMark\n\n\nHash\n\n\nQuestionMark\n\n\nComment(String)\n\n\nWhitespace(String)\n\n\nEndOfFile\n\n\nReserved(String)\n\n\nUnterminatedString(String)\n\n\nUnterminatedBytes(BitArray)\n\n\nUnexpectedGrapheme(String)\n\n\nInvalidByteLiteral(String)\n\n\nSynonyms:\ncel/parser/lexer.Token\ncel/parser/lexer Token","url":"cel/parser/lexer.html#Token"},{"doc":"cel/parser/lexer","title":"discard_comments","content":"pub fn discard_comments(lexer: Lexer) -&gt; Lexer\n\nSynonyms:\ncel/parser/lexer.discard_comments\ncel/parser/lexer discard_comments","url":"cel/parser/lexer.html#discard_comments"},{"doc":"cel/parser/lexer","title":"discard_whitespace","content":"pub fn discard_whitespace(lexer: Lexer) -&gt; Lexer\n\nSynonyms:\ncel/parser/lexer.discard_whitespace\ncel/parser/lexer discard_whitespace","url":"cel/parser/lexer.html#discard_whitespace"},{"doc":"cel/parser/lexer","title":"lex","content":"pub fn lex(lexer: Lexer) -&gt; List(#(Token, Position))\n\nSynonyms:\ncel/parser/lexer.lex\ncel/parser/lexer lex","url":"cel/parser/lexer.html#lex"},{"doc":"cel/parser/lexer","title":"new","content":"pub fn new(source: String) -&gt; Lexer\n\nSynonyms:\ncel/parser/lexer.new\ncel/parser/lexer new","url":"cel/parser/lexer.html#new"},{"doc":"cel/parser/lexer","title":"cel/parser/lexer","content":"","url":"cel/parser/lexer.html"},{"doc":"cel/parser","title":"Arithmetic","content":"pub type Arithmetic {\n  Add\n  Sub\n  Mul\n  Div\n  Mod\n}\n\nAdd\n\n\nSub\n\n\nMul\n\n\nDiv\n\n\nMod\n\n\nSynonyms:\ncel/parser.Arithmetic\ncel/parser Arithmetic","url":"cel/parser.html#Arithmetic"},{"doc":"cel/parser","title":"Atom","content":"pub type Atom {\n  Int(Int)\n  UInt(Int)\n  Float(Float)\n  String(String)\n  Bytes(BitArray)\n  Bool(Bool)\n  Null\n}\n\nInt(Int)\n\n\nUInt(Int)\n\n\nFloat(Float)\n\n\nString(String)\n\n\nBytes(BitArray)\n\n\nBool(Bool)\n\n\nNull\n\n\nSynonyms:\ncel/parser.Atom\ncel/parser Atom","url":"cel/parser.html#Atom"},{"doc":"cel/parser","title":"BinaryOp","content":"pub type BinaryOp {\n  Arithmetic(Arithmetic)\n  Relation(Relation)\n  Logical(Logical)\n}\n\nArithmetic(Arithmetic)\n\n\nRelation(Relation)\n\n\nLogical(Logical)\n\n\nSynonyms:\ncel/parser.BinaryOp\ncel/parser BinaryOp","url":"cel/parser.html#BinaryOp"},{"doc":"cel/parser","title":"Error","content":"pub type Error {\n  UnexpectedEndOfFile\n  UnexpectedSourceStr(String, Int)\n  UnexpectedToken(t.Token, Int)\n  DeadEnd(Tokens)\n}\n\nUnexpectedEndOfFile\n\n\nUnexpectedSourceStr(String, Int)\n\n\nUnexpectedToken(t.Token, Int)\n\n\nDeadEnd(Tokens)\n\n\nSynonyms:\ncel/parser.Error\ncel/parser Error","url":"cel/parser.html#Error"},{"doc":"cel/parser","title":"Expression","content":"pub type Expression {\n  BinaryOperation(ExpressionData, BinaryOp, ExpressionData)\n  Unary(UnaryOp, ExpressionData)\n  Ternary(ExpressionData, ExpressionData, ExpressionData)\n  List(List(ExpressionData))\n  Map(List(#(ExpressionData, ExpressionData)))\n  Member(ExpressionData, Member)\n  FunctionCall(\n    String,\n    option.Option(ExpressionData),\n    List(ExpressionData),\n  )\n  Atom(Atom)\n  Ident(String)\n}\n\nBinaryOperation(ExpressionData, BinaryOp, ExpressionData)\n\n\nUnary(UnaryOp, ExpressionData)\n\n\nTernary(ExpressionData, ExpressionData, ExpressionData)\n\n\nList(List(ExpressionData))\n\n\nMap(List(#(ExpressionData, ExpressionData)))\n\n\nMember(ExpressionData, Member)\n\n\nFunctionCall(\n  String,\n  option.Option(ExpressionData),\n  List(ExpressionData),\n)\n\n\nAtom(Atom)\n\n\nIdent(String)\n\n\nSynonyms:\ncel/parser.Expression\ncel/parser Expression","url":"cel/parser.html#Expression"},{"doc":"cel/parser","title":"ExpressionData","content":"pub opaque type ExpressionData\n\n\nSynonyms:\ncel/parser.ExpressionData\ncel/parser ExpressionData","url":"cel/parser.html#ExpressionData"},{"doc":"cel/parser","title":"Logical","content":"pub type Logical {\n  And\n  Or\n}\n\nAnd\n\n\nOr\n\n\nSynonyms:\ncel/parser.Logical\ncel/parser Logical","url":"cel/parser.html#Logical"},{"doc":"cel/parser","title":"Member","content":"pub type Member {\n  Attribute(String)\n  Index(ExpressionData)\n}\n\nAttribute(String)\n\n\nIndex(ExpressionData)\n\n\nSynonyms:\ncel/parser.Member\ncel/parser Member","url":"cel/parser.html#Member"},{"doc":"cel/parser","title":"Relation","content":"pub type Relation {\n  LessThan\n  LessThanEq\n  GreaterThan\n  GreaterThanEq\n  Equals\n  NotEquals\n  In\n}\n\nLessThan\n\n\nLessThanEq\n\n\nGreaterThan\n\n\nGreaterThanEq\n\n\nEquals\n\n\nNotEquals\n\n\nIn\n\n\nSynonyms:\ncel/parser.Relation\ncel/parser Relation","url":"cel/parser.html#Relation"},{"doc":"cel/parser","title":"UnaryOp","content":"pub type UnaryOp {\n  Not\n  UnarySub\n}\n\nNot\n\n\nUnarySub\n\n\nSynonyms:\ncel/parser.UnaryOp\ncel/parser UnaryOp","url":"cel/parser.html#UnaryOp"},{"doc":"cel/parser","title":"expr","content":"pub fn expr(data: ExpressionData) -&gt; Expression\n\nSynonyms:\ncel/parser.expr\ncel/parser expr","url":"cel/parser.html#expr"},{"doc":"cel/parser","title":"id","content":"pub fn id(data: ExpressionData) -&gt; Int\n\nSynonyms:\ncel/parser.id\ncel/parser id","url":"cel/parser.html#id"},{"doc":"cel/parser","title":"parse","content":"pub fn parse(source: String) -&gt; Result(ExpressionData, Error)\n\nSynonyms:\ncel/parser.parse\ncel/parser parse","url":"cel/parser.html#parse"},{"doc":"cel/parser","title":"with_id","content":"pub fn with_id(expr: Expression, id: Int) -&gt; ExpressionData\n You probably don&#39;t want to use this. Only used for testing\nSynonyms:\ncel/parser.with_id\ncel/parser with_id","url":"cel/parser.html#with_id"},{"doc":"cel/parser","title":"cel/parser","content":"","url":"cel/parser.html"},{"doc":"cel/interpreter/value","title":"Key","content":"pub type Key {\n  KeyInt(Int)\n  KeyUInt(Int)\n  KeyBool(Bool)\n  KeyString(String)\n}\n\nKeyInt(Int)\n\n\nKeyUInt(Int)\n\n\nKeyBool(Bool)\n\n\nKeyString(String)\n\n\nSynonyms:\ncel/interpreter/value.Key\ncel/interpreter/value Key","url":"cel/interpreter/value.html#Key"},{"doc":"cel/interpreter/value","title":"Value","content":"pub type Value {\n  List(List(Value))\n  Map(dict.Dict(Key, Value))\n  Function(String, List(Value))\n  Int(Int)\n  UInt(Int)\n  Float(Float)\n  String(String)\n  Bytes(BitArray)\n  Bool(Bool)\n  Null\n}\n\nList(List(Value))\n\n\nMap(dict.Dict(Key, Value))\n\n\nFunction(String, List(Value))\n\n\nInt(Int)\n\n\nUInt(Int)\n\n\nFloat(Float)\n\n\nString(String)\n\n\nBytes(BitArray)\n\n\nBool(Bool)\n\n\nNull\n\n\nSynonyms:\ncel/interpreter/value.Value\ncel/interpreter/value Value","url":"cel/interpreter/value.html#Value"},{"doc":"cel/interpreter/value","title":"decode","content":"pub fn decode(\n  value input: Dynamic,\n) -&gt; Result(Value, List(DecodeError))\n\nSynonyms:\ncel/interpreter/value.decode\ncel/interpreter/value decode","url":"cel/interpreter/value.html#decode"},{"doc":"cel/interpreter/value","title":"from_atom","content":"pub fn from_atom(atom: Atom) -&gt; Value\n\nSynonyms:\ncel/interpreter/value.from_atom\ncel/interpreter/value from_atom","url":"cel/interpreter/value.html#from_atom"},{"doc":"cel/interpreter/value","title":"key_from_value","content":"pub fn key_from_value(value: Value) -&gt; Result(Key, Nil)\n\nSynonyms:\ncel/interpreter/value.key_from_value\ncel/interpreter/value key_from_value","url":"cel/interpreter/value.html#key_from_value"},{"doc":"cel/interpreter/value","title":"cel/interpreter/value","content":"","url":"cel/interpreter/value.html"},{"doc":"cel/interpreter/type_","title":"Reference","content":"pub type Reference {\n  Constant(value: Value)\n  Variable(name: List(String))\n  Call(name: String)\n}\n A reference in an expression\nConstant(value: Value)\n A constant in an expression\n\nVariable(name: List(String))\n The fully qualified name of the variable as a list\n\nCall(name: String)\n The fully qualified name of the function as a list\n\nSynonyms:\ncel/interpreter/type_.Reference\ncel/interpreter/type_ Reference","url":"cel/interpreter/type_.html#Reference"},{"doc":"cel/interpreter/type_","title":"ReferenceMap","content":"pub type ReferenceMap {\n  ReferenceMap(dict.Dict(Int, Reference))\n}\n A [`ReferenceMap`] contains references to constants, variables, and function calls of\n an expression. Each reference is stored by their Expression ID as the key.\nReferenceMap(dict.Dict(Int, Reference))\n\n\nSynonyms:\ncel/interpreter/type_.ReferenceMap\ncel/interpreter/type_ ReferenceMap","url":"cel/interpreter/type_.html#ReferenceMap"},{"doc":"cel/interpreter/type_","title":"Type","content":"pub type Type {\n  DynamicT\n  ListT(Type)\n  MapT(Type, Type)\n  FunctionT(Type, List(Type))\n  IntT\n  UIntT\n  FloatT\n  StringT\n  BytesT\n  BoolT\n  NullT\n}\n A CEL type\nDynamicT\n\n\nListT(Type)\n\n\nMapT(Type, Type)\n\n\nFunctionT(Type, List(Type))\n\n\nIntT\n\n\nUIntT\n\n\nFloatT\n\n\nStringT\n\n\nBytesT\n\n\nBoolT\n\n\nNullT\n\n\nSynonyms:\ncel/interpreter/type_.Type\ncel/interpreter/type_ Type","url":"cel/interpreter/type_.html#Type"},{"doc":"cel/interpreter/type_","title":"functions","content":"pub fn functions(map: ReferenceMap) -&gt; List(String)\n Lists all the functions in the reference map\nSynonyms:\ncel/interpreter/type_.functions\ncel/interpreter/type_ functions","url":"cel/interpreter/type_.html#functions"},{"doc":"cel/interpreter/type_","title":"kind","content":"pub fn kind(value: Value) -&gt; Type\n Get the kind of type of a value. Will be the same as types for primitive types.\n For collections, the inner type(s) will be set to [`DynamicT`]\nSynonyms:\ncel/interpreter/type_.kind\ncel/interpreter/type_ kind","url":"cel/interpreter/type_.html#kind"},{"doc":"cel/interpreter/type_","title":"references","content":"pub fn references(expr: ExpressionData) -&gt; ReferenceMap\n Collects all constants, variables and functions used in an expression along with\n their expression ID into a reference map.\nSynonyms:\ncel/interpreter/type_.references\ncel/interpreter/type_ references","url":"cel/interpreter/type_.html#references"},{"doc":"cel/interpreter/type_","title":"variables","content":"pub fn variables(map: ReferenceMap) -&gt; List(List(String))\n Lists all the variables in the reference map\nSynonyms:\ncel/interpreter/type_.variables\ncel/interpreter/type_ variables","url":"cel/interpreter/type_.html#variables"},{"doc":"cel/interpreter/type_","title":"cel/interpreter/type_","content":"","url":"cel/interpreter/type_.html"},{"doc":"cel/interpreter/error","title":"ContextError","content":"pub type ContextError {\n  UnknownIdentifier(String)\n  UnknownFunction(String)\n  NoSuchKey(parser.Member)\n  InvalidMemberParent(parent_type: Type, member: parser.Member)\n  Decode(dynamic.DecodeErrors)\n}\n\nUnknownIdentifier(String)\n\n\nUnknownFunction(String)\n\n\nNoSuchKey(parser.Member)\n\n\nInvalidMemberParent(parent_type: Type, member: parser.Member)\n\n\nDecode(dynamic.DecodeErrors)\n\n\nSynonyms:\ncel/interpreter/error.ContextError\ncel/interpreter/error ContextError","url":"cel/interpreter/error.html#ContextError"},{"doc":"cel/interpreter/error","title":"ExecutionError","content":"pub type ExecutionError {\n  ContextError(ContextError)\n  UnsupportedBinop(Type, String, Type)\n  UnsupportedUnary(String, Type)\n  UnexpectedType(\n    expected: List(Type),\n    got: Type,\n    in_context: String,\n  )\n  InvalidValueAsKey(Value)\n  IndexOutOfBounds(size: Int, index: Int)\n  UnsupportedTernaryCondition(Type)\n  ArithmeticError\n  InvalidFunctionArgs(function: String)\n  FunctionExpectedThis(function: String)\n}\n\nContextError(ContextError)\n\n\nUnsupportedBinop(Type, String, Type)\n\n\nUnsupportedUnary(String, Type)\n\n\nUnexpectedType(\n  expected: List(Type),\n  got: Type,\n  in_context: String,\n)\n\n\nInvalidValueAsKey(Value)\n\n\nIndexOutOfBounds(size: Int, index: Int)\n\n\nUnsupportedTernaryCondition(Type)\n\n\nArithmeticError\n\n\nInvalidFunctionArgs(function: String)\n\n\nFunctionExpectedThis(function: String)\n\n\nSynonyms:\ncel/interpreter/error.ExecutionError\ncel/interpreter/error ExecutionError","url":"cel/interpreter/error.html#ExecutionError"},{"doc":"cel/interpreter/error","title":"cel/interpreter/error","content":"","url":"cel/interpreter/error.html"},{"doc":"cel/interpreter/context","title":"Callable","content":"pub type Callable {\n  Callable(\n    call: fn(FunctionContext) -&gt; Result(Value, ExecutionError),\n  )\n}\n\nCallable(\n  call: fn(FunctionContext) -&gt; Result(Value, ExecutionError),\n)\n\n\nSynonyms:\ncel/interpreter/context.Callable\ncel/interpreter/context Callable","url":"cel/interpreter/context.html#Callable"},{"doc":"cel/interpreter/context","title":"Context","content":"pub type Context {\n  Root(\n    variables: Dict(String, Value),\n    functions: Dict(String, Callable),\n  )\n  Child(variables: Dict(String, Value), parent: Context)\n}\n\nRoot(\n  variables: Dict(String, Value),\n  functions: Dict(String, Callable),\n)\n\n\nChild(variables: Dict(String, Value), parent: Context)\n\n\nSynonyms:\ncel/interpreter/context.Context\ncel/interpreter/context Context","url":"cel/interpreter/context.html#Context"},{"doc":"cel/interpreter/context","title":"FunctionContext","content":"pub type FunctionContext {\n  FunctionContext(\n    name: String,\n    this: Option(Value),\n    ctx: Context,\n    args: List(Expression),\n  )\n}\n\nFunctionContext(\n  name: String,\n  this: Option(Value),\n  ctx: Context,\n  args: List(Expression),\n)\n\n\nSynonyms:\ncel/interpreter/context.FunctionContext\ncel/interpreter/context FunctionContext","url":"cel/interpreter/context.html#FunctionContext"},{"doc":"cel/interpreter/context","title":"empty","content":"pub fn empty() -&gt; Context\n\nSynonyms:\ncel/interpreter/context.empty\ncel/interpreter/context empty","url":"cel/interpreter/context.html#empty"},{"doc":"cel/interpreter/context","title":"insert_function","content":"pub fn insert_function(\n  ctx: Context,\n  name: String,\n  func: fn(FunctionContext) -&gt; Result(Value, ExecutionError),\n) -&gt; Context\n\nSynonyms:\ncel/interpreter/context.insert_function\ncel/interpreter/context insert_function","url":"cel/interpreter/context.html#insert_function"},{"doc":"cel/interpreter/context","title":"insert_variable","content":"pub fn insert_variable(\n  ctx: Context,\n  name: String,\n  value: Value,\n) -&gt; Context\n\nSynonyms:\ncel/interpreter/context.insert_variable\ncel/interpreter/context insert_variable","url":"cel/interpreter/context.html#insert_variable"},{"doc":"cel/interpreter/context","title":"new_inner","content":"pub fn new_inner(ctx: Context) -&gt; Context\n\nSynonyms:\ncel/interpreter/context.new_inner\ncel/interpreter/context new_inner","url":"cel/interpreter/context.html#new_inner"},{"doc":"cel/interpreter/context","title":"resolve_function","content":"pub fn resolve_function(\n  ctx: Context,\n  name: String,\n) -&gt; Result(Callable, ContextError)\n\nSynonyms:\ncel/interpreter/context.resolve_function\ncel/interpreter/context resolve_function","url":"cel/interpreter/context.html#resolve_function"},{"doc":"cel/interpreter/context","title":"resolve_variable","content":"pub fn resolve_variable(\n  ctx: Context,\n  name: String,\n) -&gt; Result(Value, ContextError)\n\nSynonyms:\ncel/interpreter/context.resolve_variable\ncel/interpreter/context resolve_variable","url":"cel/interpreter/context.html#resolve_variable"},{"doc":"cel/interpreter/context","title":"try_insert_variable","content":"pub fn try_insert_variable(\n  ctx: Context,\n  name: String,\n  input: Dynamic,\n) -&gt; Result(Context, ContextError)\n\nSynonyms:\ncel/interpreter/context.try_insert_variable\ncel/interpreter/context try_insert_variable","url":"cel/interpreter/context.html#try_insert_variable"},{"doc":"cel/interpreter/context","title":"cel/interpreter/context","content":"","url":"cel/interpreter/context.html"},{"doc":"cel/interpreter/evaluate","title":"evaluate_expr","content":"pub fn evaluate_expr(\n  expr: Expression,\n  ctx: Context,\n) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/evaluate.evaluate_expr\ncel/interpreter/evaluate evaluate_expr","url":"cel/interpreter/evaluate.html#evaluate_expr"},{"doc":"cel/interpreter/evaluate","title":"cel/interpreter/evaluate","content":"","url":"cel/interpreter/evaluate.html"},{"doc":"cel/interpreter/function","title":"all","content":"pub fn all(ftx: FunctionContext) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.all\ncel/interpreter/function all","url":"cel/interpreter/function.html#all"},{"doc":"cel/interpreter/function","title":"exists","content":"pub fn exists(\n  ftx: FunctionContext,\n) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.exists\ncel/interpreter/function exists","url":"cel/interpreter/function.html#exists"},{"doc":"cel/interpreter/function","title":"exists_one","content":"pub fn exists_one(\n  ftx: FunctionContext,\n) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.exists_one\ncel/interpreter/function exists_one","url":"cel/interpreter/function.html#exists_one"},{"doc":"cel/interpreter/function","title":"filter","content":"pub fn filter(\n  ftx: FunctionContext,\n) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.filter\ncel/interpreter/function filter","url":"cel/interpreter/function.html#filter"},{"doc":"cel/interpreter/function","title":"has","content":"pub fn has(ftx: FunctionContext) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.has\ncel/interpreter/function has","url":"cel/interpreter/function.html#has"},{"doc":"cel/interpreter/function","title":"map","content":"pub fn map(ftx: FunctionContext) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.map\ncel/interpreter/function map","url":"cel/interpreter/function.html#map"},{"doc":"cel/interpreter/function","title":"size","content":"pub fn size(\n  ftx: FunctionContext,\n) -&gt; Result(Value, ExecutionError)\n\nSynonyms:\ncel/interpreter/function.size\ncel/interpreter/function size","url":"cel/interpreter/function.html#size"},{"doc":"cel/interpreter/function","title":"cel/interpreter/function","content":"","url":"cel/interpreter/function.html"},{"doc":"cel/interpreter","title":"Program","content":"pub opaque type Program\n An executable CEL program\n\nSynonyms:\ncel/interpreter.Program\ncel/interpreter Program","url":"cel/interpreter.html#Program"},{"doc":"cel/interpreter","title":"default_context","content":"pub fn default_context() -&gt; Context\n Create a [`context.Context`] with the default functions.\n The default functions are:\n - *filter*: Filters out each element that doesn&#39;t meet the predicate provided. Example: `[1,2,3].filter(x, x % 2 == 0)` will result in `[2]`.\n - *map*: Maps each element with the provided function. Example: `[1,2,3].map(x, x * 2)` will result in `[2,4,6]`.\n - *all*: Checks whether the provided predicate is true for all of the elements. Example: `[1,2,3].all(x, x &gt; 1)` will result in `false`.\n - *size*: Returns the amount of elements of the container argument provided. Example: `size([1,2,3])` will result in `3`.\n - *has*: Checks whether an identifier has been set in the execution context. Example: `has(doesnt.exist)` will result in `false` for an empty context.\n - *exists*: Similar to `all` but will be true if at least one predicate returns true. Example: `[1,2,3].exists(x, x &gt; 1)` will result in `true`.\n - *exists_one*: Similar to `exists`. Will return true if exactly one of the predicates is true. Example: `[1,2,3].exists_one(x, x &gt; 1)` will result in `false`.\nSynonyms:\ncel/interpreter.default_context\ncel/interpreter default_context","url":"cel/interpreter.html#default_context"},{"doc":"cel/interpreter","title":"execute","content":"pub fn execute(\n  program: Program,\n  ctx: Context,\n) -&gt; Result(Value, ExecutionError)\n Execute a CEL program\nSynonyms:\ncel/interpreter.execute\ncel/interpreter execute","url":"cel/interpreter.html#execute"},{"doc":"cel/interpreter","title":"new","content":"pub fn new(from source: String) -&gt; Result(Program, Error)\n Parses the source CEL expression into an executable CEL program\nSynonyms:\ncel/interpreter.new\ncel/interpreter new","url":"cel/interpreter.html#new"},{"doc":"cel/interpreter","title":"cel/interpreter","content":"","url":"cel/interpreter.html"}]);